@@ -14,7 +14,7 @@ require Exporter;
 our @ISA = qw(Exporter);
 
 our %EXPORT_TAGS = ( 'all' => [ qw(
-	get_asn_info get_as_description get_soa_contact get_ipwi_contacts
+	get_asn_info get_peer_info get_as_description get_soa_contact get_ipwi_contacts
 	get_rdns get_dnsbl_listing get_ip_country get_asn_country
 	get_abusenet_contact is_ip get_as_company get_domain
 ) ] );
@@ -29,11 +29,11 @@ Net::Abuse::Utils - Routines useful for processing network abuse
 
 =head1 VERSION
 
-This documentation refers to Net::Abuse::Utils version 0.11.
+This documentation refers to Net::Abuse::Utils version 0.12.
 
 =cut
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 $VERSION = eval $VERSION;
 
 # memoize('_return_rr');
@@ -112,6 +112,11 @@ sub get_ipwi_contacts {
     my @addresses;
     my %unique_addresses;
 
+    # work-around for the new way arin works
+    # it doesn't like networks very well.
+    my @bits = split(/\//,$ip);
+    $ip = $bits[0] if($#bits > 0);
+    
     my $response = whoisip_query($ip);
 
     # whoisip_query returns array ref if not found
@@ -132,11 +137,14 @@ sub get_asn_info {
     my $lookup    = _reverse_ip($ip) . '.origin.asn.cymru.com';
     my @origin_as = _return_rr($lookup, 'TXT', 2) or return;
     
+    # un-comment for testing
+    #push(@origin_as,'20738 | 212.67.192.0/24 | GB | ripencc | 1999-05-12');
+
     # 23028 | 216.90.108.0/24 | US | arin | 1998-09-25
     # 701 1239 3549 3561 7132 | 216.90.108.0/24 | US | arin | 1998-09-25
  
     my $smallest_netmask = 0;
-    my ($smallest_asn, %data_for_asn);
+    my ($smallest_prefix, %data_for_asn);
 
     # surely there is a better way to do this, at least the split
     # fields are stored so they don't have to be split again ;)
@@ -152,15 +160,46 @@ sub get_asn_info {
 
         my $asn = $fields[0];
                  
-        $data_for_asn{$asn} = [ @fields ];
+        $data_for_asn{$fields[1]} = [@fields];
         
         if ($network[1] > $smallest_netmask) {
             $smallest_netmask = $network[1];
-            $smallest_asn     = $asn;
+            $smallest_prefix  = $fields[1];
         }
     }
 
-    return map { _strip_whitespace($_) } @{ $data_for_asn{$smallest_asn} };
+    return map { _strip_whitespace($_) } @{ $data_for_asn{$smallest_prefix} };
+}
+
+sub get_peer_info {
+    my $ip = shift;
+
+    my $lookup    = _reverse_ip($ip) . '.peer.asn.cymru.com';
+    my @origin_as = _return_rr($lookup, 'TXT', 2) or return;
+    
+    my $return = [];
+    foreach my $as (@origin_as){
+        my @peers = split(/\s\|\s/,$as);
+        my %hash = (
+            prefix  => $peers[1],
+            cc      => $peers[2],
+            rir     => $peers[3],
+            date    => $peers[4],
+        );
+        my @asns = split(/\s/,$peers[0]);
+        foreach (@asns){ 
+            $hash{'asn'} = $_;
+            push(@$return,{
+                prefix  => $peers[1],
+                cc      => $peers[2],
+                rir     => $peers[3],
+                date    => $peers[4],
+                asn     => $_,
+            });
+        }
+    }
+    return(@$return) if wantarray;    
+    return($return);
 }
 
 sub get_as_description {
@@ -310,6 +349,11 @@ of them into your namespace with the C<:all> tag.
 Returns a list containing (ASN, Network/Mask, CC code, RIR, modified date)
 for the network announcing C<IP>.
 
+=item get_peer_info ( IP )
+
+Returns an array of hash references containing (ASN, Network/Mask, CC code, RIR, modified date)
+for the peers of the network announcing C<IP>.
+
 =item get_as_description ( ASN )
 
 Returns the AS description for C<ASN>. 
@@ -396,9 +440,10 @@ Rik Rose, Jon Honeycutt, Brandon Hale, TJ Fontaine, A. Pagaltzis, and
 Heidi Greb all provided invaluable input during the development of this
 module.
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Michael Greb (mgreb@linode.com)
+Michael Greb, <mgreb@linode.com>
+Wes Young, <wes@barely3am.com>
 
 =head1 SEE ALSO
 
@@ -2230,4 +2275,4 @@ zhitomir.ua
 zj.cn
 zlg.br
 zp.ua
-zt.ua
\ No newline at end of file
+zt.ua
